@@ -1,14 +1,16 @@
 # -*- coding: utf-8 -*-
 
-from importlib import import_module
 from django.conf import settings
-
+from importlib import import_module
 
 modules = list()
 
 modules.append(import_module('django.template.defaultfilters'))
 for item in settings.TEMPLATES:
-    for builtins in item['OPTIONS']['builtins']:
+    for builtins in [
+                'djangoplus.templatetags',
+                'djangoplus.ui.components.paginator.templatetags'
+            ]:
         modules.append(import_module(builtins))
 
 
