@@ -1,30 +1,40 @@
 # -*- coding: utf-8 -*-
+
 import copy
-from djangoplus.utils import permissions
+from collections import OrderedDict
 from djangoplus.ui.components import forms
 from django.db.models.aggregates import Sum
 from djangoplus.utils.tabulardata import tolist
 from djangoplus.utils.formatter import normalyze
-from djangoplus.ui import RequestComponent, ComponentHasResponseException
+from djangoplus.ui.components.forms import factory
+from django.core.exceptions import ValidationError
+from django.utils.translation import ugettext as _
+from djangoplus.utils import permissions, should_add_action, get_role_values_for_condition
+from djangoplus.ui.components.navigation.breadcrumbs import httprr
+from djangoplus.ui.components import Component, ComponentHasResponseException
+from djangoplus.utils.http import CsvResponse, XlsResponse, ReportResponse, return_response
 from djangoplus.ui.components.navigation.dropdown import ModelDropDown, GroupDropDown
-from djangoplus.utils.http import CsvResponse, XlsResponse, ReportResponse, mobile
-from djangoplus.utils.metadata import get_metadata, get_field, get_fiendly_name, should_filter_or_display, getattr2
+from djangoplus.utils.metadata import get_metadata, get_field, get_fiendly_name, should_filter_or_display, getattr2, \
+    get_parameters_names, count_parameters_names
 
 
-class Paginator(RequestComponent):
+class Paginator(Component):
     def __init__(self, request, qs, title=None, list_display=None, list_filter=None, search_fields=None,
                  list_per_page=25, list_subsets=None, exclude=None, relation=None, readonly=False, is_list_view=False,
-                 help_text=None, url=None, template='datagrid.html'):
+                 help_text=None, url=None, template=None, uid=None):
 
         super(Paginator, self).__init__(is_list_view and '_' or abs(hash(title)), request)
-
-        self.qs = qs.all()
+        if relation:
+            qs = qs.model.objects.filter(pk__in=qs.values_list('pk', flat=True))
+        else:
+            qs = qs.all()
+        self.qs = qs
         self.title = title
         self.list_display = list_display
         self.list_filter = list_filter
         self.search_fields = search_fields
         self.list_per_page = list_per_page
-        self.list_subsets = list_subsets
+        self.subsets = None
         self.exclude = exclude
         self.relation = relation
         self.readonly = readonly
@@ -32,11 +42,12 @@ class Paginator(RequestComponent):
         self.icon = get_metadata(qs.model, 'icon', None)
         self.list_total = get_metadata(qs.model, 'list_total', None)
         self.ordering = get_metadata(qs.model, 'ordering', None)
-        self.template = get_metadata(self.qs.model, 'list_template', template)
+        self.template = template or get_metadata(self.qs.model, 'list_template', 'datagrid.html')
         self.help_text = help_text
         self.url = url
+        self.display_checkboxes = False
+        self.action_names = None
 
-        self.subset_actions = []
         self.filters = []
         self.pagination = ''
 
@@ -46,6 +57,10 @@ class Paginator(RequestComponent):
         # tabs
         self.tabs = []
         self.current_tab = self._get_from_request('tab', None)
+        if list_subsets is not None:
+            self.subsets = OrderedDict()
+            for subset_name in list_subsets:
+                self.subsets[subset_name] = {}
         self._load_tabs()
 
         # list display
@@ -56,10 +71,9 @@ class Paginator(RequestComponent):
         self._configure_column_names()
 
         # drop down
-        self.paginator_dropdown = GroupDropDown(request)
-        self.subset_dropdown = GroupDropDown(request)
+        self.class_dropdown = GroupDropDown(request, style='class-action')
+        self.queryset_dropdown = GroupDropDown(request, style='queryset-action disabled')
         self.drop_down = ModelDropDown(request, qs.model)
-        self.mobile = mobile(self.request)
 
         if hasattr(self.qs, 'permission_map'):
             self.permission_map = self.qs.permission_map
@@ -144,7 +158,7 @@ class Paginator(RequestComponent):
                             qs = field.remote_field.model.objects.get_queryset().filter(pk__in=pks)
                         empty_label = ''
 
-                        form.fields[form_field_name] = forms.ModelChoiceField(qs, label=normalyze(field.verbose_name), initial=initial, empty_label=empty_label, required=False, lazy=True, ignore_lookup=True, minimum_input_length=0)
+                        form.fields[form_field_name] = forms.ModelChoiceField(qs, label=normalyze(field.verbose_name), initial=initial, empty_label=empty_label, required=False, lazy=False, ignore_lookup=True, minimum_input_length=0)
                     form.fields[form_field_name].widget.attrs['data-placeholder'] = field.verbose_name
                     if initial:
                         label = form.fields[form_field_name].label
@@ -164,54 +178,206 @@ class Paginator(RequestComponent):
             ids = 0 not in ids and ids or None
         return ids
 
-    def add_action(self, label, url, css, icon=None):
-        self.paginator_dropdown.add_action(label, url, 'global-action {}'.format(css), icon, label)
+    def add_action(self, label, url, css, icon=None, category=None):
+        self.class_dropdown.add_action(label, url, 'class-action {}'.format(css), icon, category)
 
-    def add_subset_action(self, label, url, css, icon=None, subset=None):
+    def add_queryset_action(self, label, url, css, icon=None, category=None, subset=None):
         if not subset or self.get_tab() == subset:
             if not self.mobile:
-                self.subset_dropdown.add_action(label, url, 'subset-action disabled {}'.format(css), icon, label)
+                self.queryset_dropdown.add_action(label, url, 'queryset-action disabled {}'.format(css), icon, category)
 
-    def add_actions(self):
+    def load_actions(self, action_names=None):
+
+        from djangoplus.cache import CACHE
         export_url = self.request.get_full_path()
         list_csv = get_metadata(self.qs.model, 'list_csv')
         list_xls = get_metadata(self.qs.model, 'list_xls')
         log = get_metadata(self.qs.model, 'log')
         app_label = get_metadata(self.qs.model, 'app_label')
         list_pdf = get_metadata(self.qs.model, 'list_pdf')
+        self.action_names = action_names or []
 
-        if list_csv:
-            export_url = '?' in export_url and '{}&export=csv'.format(export_url) or '{}?export=csv'.format(export_url)
-            self.add_action('Exportar CSV', export_url, 'ajax', 'fa-table')
+        subset = self.subsets and list(self.subsets.keys())[0] or None
+        subsetp = self.request.GET.get('tab{}'.format(self.request.GET.get('tid')))
+        subset_name = subsetp or subset
 
-        if list_xls:
-            export_url = '?' in export_url and '{}&export=excel'.format(export_url) or '{}?export=excel'.format(export_url)
-            self.add_action('Exportar Excel', export_url, 'ajax', 'fa-file-excel-o')
+        # class actions defined in the manager
+        if self.qs.model in CACHE['CLASS_ACTIONS']:
+            for group in CACHE['CLASS_ACTIONS'][self.qs.model]:
+                for view_name in CACHE['CLASS_ACTIONS'][self.qs.model][group]:
+                    _action = CACHE['CLASS_ACTIONS'][self.qs.model][group][view_name]
+                    action_verbose_name = _action['verbose_name']
+                    action_inline = _action['inline']
+                    action_subsets = _action['subsets']
+                    action_can_execute = _action['can_execute']
+                    action_category = _action['group']
+                    action_expose = _action['expose']
+                    action_style = _action['style'] or 'popup'
+                    if 'popup' not in action_style:
+                        action_style = '{} popup'.format(action_style)
+                    if self.is_list_view or view_name in self.action_names:
+                        add_action = should_add_action(action_inline, action_subsets, subset_name)
+                        add_action = add_action and (True in action_expose or 'web' in action_expose)
+                        if add_action and permissions.check_group_or_permission(self.request, action_can_execute):
+                            func = getattr(self.qs, view_name)
+                            ignore_pdf = False
+                            if count_parameters_names(func) > 0:
+                                action_style = action_style.replace('pdf', '')
+                                ignore_pdf = True
+                            char = '?' in self.request.get_full_path() and '&' or '?'
+                            url = '{}{}{}'.format(self.request.get_full_path(), char, '{}='.format(view_name))
+                            self.add_action(action_verbose_name, url, action_style, None, action_category)
+                            if view_name in self.request.GET:
+                                self._proccess_request(func, _action, ignore_pdf)
 
-        if log:
-            log_url = '/log/{}/{}/'.format(app_label, self.qs.model.__name__.lower())
-            if self.request.user.has_perm('admin.list_log'):
-                self.add_action('Visualizar Log', log_url, 'ajax', 'fa-history')
+        # subset actions defined in the manager
+        if self.qs.model in CACHE['QUERYSET_ACTIONS']:
+            for group in CACHE['QUERYSET_ACTIONS'][self.qs.model]:
+                for view_name in CACHE['QUERYSET_ACTIONS'][self.qs.model][group]:
+                    _action = CACHE['QUERYSET_ACTIONS'][self.qs.model][group][view_name]
+                    action_verbose_name = _action['verbose_name']
+                    action_can_execute = _action['can_execute']
+                    action_inline = _action['inline']
+                    action_icon = _action['icon']
+                    action_subsets = _action['subsets']
+                    action_condition = _action['condition']
+                    action_source = _action['source']
+                    action_category = _action['group']
+                    action_expose = _action['expose']
+                    action_style = _action['style'] or 'popup'
+                    if 'popup' not in action_style:
+                        action_style = '{} popup'.format(action_style)
+                    if view_name in self.action_names or (self.is_list_view and action_inline) or (self.current_tab in action_subsets):
+                        add_action = should_add_action(action_inline, action_subsets, subset_name)
+                        add_action = add_action and (True in action_expose or 'web' in action_expose)
+                        if add_action and permissions.check_group_or_permission(self.request, action_can_execute):
+                            self.display_checkboxes = self.display_checkboxes or not action_source == 'view'
+                            func = getattr(self.qs, view_name)
+                            ignore_pdf = False
+                            char = '?' in self.request.get_full_path() and '&' or '?'
+                            url = '{}{}{}'.format(self.request.get_full_path(), char, '{}='.format(view_name))
+                            if count_parameters_names(func) > 0:
+                                ignore_pdf = True
+                                action_style = action_style.replace('pdf', '')
+                            self.add_queryset_action(
+                                action_verbose_name, url, action_style, action_icon, action_category, action_condition
+                            )
 
-        if list_pdf:
-            pdf_url = '?' in export_url and '{}&export=pdf'.format(export_url) or '{}?export=pdf'.format(export_url)
-            self.add_action('Imprimir', pdf_url, 'ajax', 'fa-print')
+                            if view_name in self.request.GET:
+                                ids = self.get_selected_ids()
+                                if ids:
+                                    qs = self.get_queryset(paginate=False).filter(id__in=ids)
+                                else:
+                                    break
+                                func = getattr(qs, view_name)
+                                self._proccess_request(func, _action, ignore_pdf)
 
-        subclasses = self.qs.model.__subclasses__()
+        # class actions defined in views module
+        if self.qs.model in CACHE['CLASS_VIEW_ACTIONS']:
+            for group in CACHE['CLASS_VIEW_ACTIONS'][self.qs.model]:
+                for view_name in CACHE['CLASS_VIEW_ACTIONS'][self.qs.model][group]:
+                    _action = CACHE['CLASS_VIEW_ACTIONS'][self.qs.model][group][view_name]
+                    action_verbose_name = _action['verbose_name']
+                    action_inline = _action['inline']
+                    action_subsets = _action['subsets']
+                    action_can_execute = _action['can_execute']
+                    action_category = _action['group']
+                    action_expose = _action['expose']
+                    action_style = _action['style'] or 'popup'
+                    url = '/{}/{}/'.format(get_metadata(self.qs.model, 'app_label'), view_name)
+                    action_style = action_style.replace('popup', '')
+                    if self.is_list_view or view_name in self.action_names:
+                        add_action = should_add_action(action_inline, action_subsets, subset_name)
+                        add_action = add_action and (True in action_expose or 'web' in action_expose)
+                        if add_action and permissions.check_group_or_permission(self.request, action_can_execute):
+                            self.add_action(action_verbose_name, url, action_style, None, action_category)
 
-        if not subclasses and not self.list_subsets and permissions.has_add_permission(self.request, self.qs.model):
-            instance = self.qs.model()
-            instance.user = self.request.user
-            if not hasattr(instance, 'can_add') or instance.can_add():
-                add_label = get_metadata(self.qs.model, 'add_label', 'Cadastrar')
-                self.add_action(add_label, '/add/{}/{}/'.format(app_label, self.qs.model.__name__.lower()), 'ajax', 'fa-plus')
+        if self.is_list_view:
+            # utility actions
+            if list_csv:
+                export_url = '?' in export_url and '{}&export=csv'.format(export_url) or '{}?export=csv'.format(export_url)
+                self.add_action('Exportar CSV', export_url, 'ajax', 'fa-table')
 
-        for subclass in subclasses:
-            app = get_metadata(subclass, 'app_label')
-            verbose_name = get_metadata(subclass, 'verbose_name')
-            cls = subclass.__name__.lower()
-            if permissions.has_add_permission(self.request, subclass):
-                self.add_action(verbose_name, '/add/{}/{}/'.format(app, cls), False, 'fa-plus')
+            if list_xls:
+                export_url = '?' in export_url and '{}&export=excel'.format(export_url) or '{}?export=excel'.format(export_url)
+                self.add_action('Exportar Excel', export_url, 'ajax', 'fa-file-excel-o')
+
+            if log:
+                log_url = '/log/{}/{}/'.format(app_label, self.qs.model.__name__.lower())
+                if self.request.user.has_perm('admin.list_log'):
+                    self.add_action('Visualizar Log', log_url, 'ajax', 'fa-history')
+
+            if list_pdf:
+                pdf_url = '?' in export_url and '{}&export=pdf'.format(export_url) or '{}?export=pdf'.format(export_url)
+                self.add_action('Imprimir', pdf_url, 'ajax', 'fa-print')
+
+            # registration action
+            subclasses = self.qs.model.__subclasses__()
+            if not subclasses and not self.subsets and permissions.has_add_permission(self.request, self.qs.model):
+                instance = self.qs.model()
+                instance._user = self.request.user
+                if not hasattr(instance, 'can_add') or instance.can_add():
+                    if self.relation:
+                        verbose_name = get_metadata(self.qs.model, 'verbose_name')
+                        add_label = get_metadata(self.qs.model, 'add_label', '{} {}'.format(_('Add'), verbose_name))
+                    else:
+                        add_label = get_metadata(self.qs.model, 'add_label', _('Add'))
+                    add_url = '/add/{}/{}/'.format(app_label, self.qs.model.__name__.lower())
+                    self.add_action(add_label, add_url, 'ajax', 'fa-plus')
+
+            for subclass in subclasses:
+                app = get_metadata(subclass, 'app_label')
+                verbose_name = get_metadata(subclass, 'verbose_name')
+                cls = subclass.__name__.lower()
+                if permissions.has_add_permission(self.request, subclass):
+                    self.add_action(verbose_name, '/add/{}/{}/'.format(app, cls), False, 'fa-plus')
+
+    def _proccess_request(self, func, _action, ignore_pdf):
+
+        form = None
+        f_return = None
+        redirect_url = None
+
+        if count_parameters_names(func) > 0:
+            form = factory.get_class_action_form(self.request, self.qs.model, _action, func)
+            if form.is_valid():
+                params = []
+                for param in get_parameters_names(func):
+                    if param in form.cleaned_data:
+                        params.append(form.cleaned_data[param])
+                try:
+                    f_return = func(*params)
+                    ignore_pdf = False
+                    if f_return is None:
+                        redirect_url = '..'
+                except ValidationError as e:
+                    form.add_error(None, str(e.message))
+        else:
+            f_return = func()
+            ignore_pdf = False
+            if f_return is None:
+                redirect_url = '.'
+
+        if redirect_url:
+            self.request.GET._mutable = True
+            if 'ids' in self.request.GET:
+                del self.request.GET['ids']
+            del self.request.GET[_action['view_name']]
+            self.request.GET._mutable = False
+            raise ComponentHasResponseException(
+                httprr(self.request, redirect_url, _action['message'])
+            )
+        else:
+            action_style = _action['style']
+            if f_return is None:
+                f_return = dict(form=form)
+                template_name = 'default.html'
+            else:
+                template_name = '{}.html'.format(_action['view_name'])
+            return_response(
+                f_return, self.request, None, action_style, template_name,
+                raise_response=True, ignore_pdf=ignore_pdf
+            )
 
     def get_total(self):
         if self.list_total:
@@ -229,7 +395,8 @@ class Paginator(RequestComponent):
             if not type (self.ordering) == tuple:
                 self.ordering = self.ordering,
             queryset = queryset.order_by(*self.ordering)
-
+        if self.get_list_filter():
+            queryset = queryset.distinct()
         if paginate:
             l = []
 
@@ -315,18 +482,22 @@ class Paginator(RequestComponent):
                 self.column_names.append(get_fiendly_name(self.qs.model, lookup, as_tuple=True))
 
     def _load_tabs(self):
-        from djangoplus.cache import loader
-        if self.list_subsets is None:
-            list_subsets = loader.subsets[self.qs.model]
-        else:
-            list_subsets = []
-            for subset_name in self.list_subsets:
-                for subset in loader.subsets[self.qs.model]:
+        from djangoplus.cache import CACHE
+
+        subsets = []
+        if self.subsets is None:
+            subsets = CACHE['SUBSETS'][self.qs.model]
+        elif self.subsets:
+            for subset_name in self.subsets.keys():
+                for subset in CACHE['SUBSETS'][self.qs.model]:
                     if subset['name'] == subset_name:
-                        list_subsets.append(subset)
+                        subsets.append(subset)
+
+        tab_qs = None
+        active_queryset = None
         create_default_tab = True
-        for subset in list_subsets:
-            tab_title = subset['title']
+        for subset in subsets:
+            tab_title = subset['verbose_name']
             tab_function = subset['function']
             tab_can_view = subset['can_view']
             tab_name = subset['name']
@@ -336,29 +507,38 @@ class Paginator(RequestComponent):
             tab_list_filter = subset['list_filter']
             tab_search_fields = subset['search_fields']
             tab_active = False
+            tab_expose = subset['expose']
+            if True not in tab_expose and 'web' not in tab_expose:
+                continue
             if permissions.check_group_or_permission(self.request, tab_can_view):
-                tab_qs = getattr(tab_function.__self__.all(self.request.user), tab_function.__func__.__name__)()
+                func = getattr(self.qs, tab_function.__func__.__name__)
+                params = get_role_values_for_condition(func, self.request.user)
+                tab_qs = func(*params)
                 tab_active = self.current_tab == tab_name
                 self.tabs.append([tab_name, tab_title, tab_qs, tab_active, tab_order, tab_help_text])
-                if (tab_active or len(list_subsets) == 1) and tab_help_text:
+                if (tab_active or len(subsets) == 1) and tab_help_text:
                     self.help_text = tab_help_text
             if tab_name == 'all':
                 create_default_tab = False
             if tab_active:
-                self.qs = tab_qs
+                active_queryset = tab_qs
                 self.drop_down = ModelDropDown(self.request, self.qs.model)
                 self.list_display = tab_list_display
                 self.list_filter = tab_list_filter
                 self.search_fields = tab_search_fields
 
         self.tabs = sorted(self.tabs, key=lambda k: k[4])
-        if self.list_subsets is None and create_default_tab:
+        if (self.subsets is None or self.relation) and create_default_tab:
             tab_title = get_metadata(self.qs.model, 'verbose_female') and 'Todas' or 'Todos'
-            tab_qs = self.original_qs
+            tab_qs = self.original_qs.all()
             tab_active = self.current_tab == 'all'
+            if tab_active:
+                active_queryset = tab_qs
             tab_order = 0
             self.tabs.insert(0, ['', tab_title, tab_qs, tab_active, tab_order, None])
-        if not self.current_tab and self.tabs:
+        if active_queryset is not None:
+            self.qs = active_queryset
+        elif self.tabs:
             self.tabs[0][3] = True
             self.qs = self.tabs[0][2]
 
@@ -430,7 +610,7 @@ class Paginator(RequestComponent):
                     pass
         if distinct:
             queryset = queryset.distinct()
-        return queryset
+        return queryset.all(self.request.user)
 
     def process_request(self):
         export = self.request.GET.get('export', None)
@@ -458,4 +638,7 @@ class Paginator(RequestComponent):
     def _get_pdf_response(self):
         self.as_pdf = True
         landscape = len(self.list_display) > 4
-        return ReportResponse(self.title, self.request, [self], landscape)
\ No newline at end of file
+        return ReportResponse(self.title, self.request, [self], landscape)
+
+    def __str__(self):
+        return self.render('base/paginator.html')
