@@ -1,8 +1,9 @@
 # -*- coding: utf-8 -*-
+
 from decimal import Decimal
-from djangoplus.ui import RequestComponent
+from djangoplus.ui.components import Component
 from django.utils.text import slugify
-from djangoplus.utils import permissions
+from djangoplus.utils import permissions, execute_and_format
 from django.template.loader import render_to_string
 from djangoplus.ui.components.paginator import Paginator
 from djangoplus.ui.components.navigation.dropdown import ModelDropDown, GroupDropDown
@@ -10,18 +11,31 @@ from djangoplus.utils.metadata import get_metadata, get_fieldsets, find_field_by
     check_condition, is_one_to_one, is_many_to_one, should_filter_or_display
 
 
-class Panel(RequestComponent):
+class Panel(Component):
 
     def __init__(self, request, title=None, text=None, icon=None):
         super(Panel, self).__init__(title, request)
         self.title = title
         self.text = text
-        self.labels = []
         self.icon = icon
 
 
-class ModelPanel(RequestComponent):
-    def __init__(self, request, obj, current_tab=None, parent=None, fieldsets=None, complete=True, readonly=False, printable=True):
+class ImagePanel(Component):
+    def __init__(self, images=(), request=None, title=None, icon=None):
+        super(ImagePanel, self).__init__(title, request)
+        self.title = title
+        self.icon = icon
+        self.images = []
+        for image in images:
+            if type(image) in (tuple, list):
+                self.images.append(image)
+            else:
+                self.images.append((image, None))
+
+
+class ModelPanel(Component):
+    def __init__(self, request, obj, current_tab=None, parent=None,
+                 fieldsets=None, complete=True, readonly=False, printable=True):
 
         super(ModelPanel, self).__init__(obj.pk, request)
 
@@ -41,9 +55,12 @@ class ModelPanel(RequestComponent):
 
         if self.complete:
             self.drop_down = ModelDropDown(self.request, type(self.obj))
-            self.drop_down.add_actions(self.obj, fieldset_title='')
+            self.drop_down.add_actions(self.obj, fieldset='')
             if self.printable:
-                self.drop_down.add_action('Imprimir', url='?pdf={}&pk='.format(self.id), css='ajax', icon='fa-print', category='Imprimir')
+                print_url = '?pdf={}&pk='.format(self.id)
+                self.drop_down.add_action(
+                    'Imprimir', url=print_url, css='ajax', icon='fa-print', category='Imprimir'
+                )
         else:
             self.drop_down = GroupDropDown(self.request)
 
@@ -62,17 +79,19 @@ class ModelPanel(RequestComponent):
             fieldset_image = info.get('image')
 
             if fieldset_actions:
-                drop_down.add_actions(self.obj, fieldset_title=title)
+                drop_down.add_actions(self.obj, fieldset=title)
 
             if 'condition' in fieldset[1]:
                 condition = fieldset[1]['condition']
                 self.obj.request = self.request
-                if not check_condition(condition, self.obj):
+                if not check_condition(self.request.user, condition, self.obj):
                     continue
 
             if '::' in title:
                 tab_name, title = title.split('::')
-                url = '/view/{}/{}/{}/{}/'.format(get_metadata(model, 'app_label'), model.__name__.lower(), self.obj.pk, slugify(tab_name))
+                url = '/view/{}/{}/{}/{}/'.format(
+                    get_metadata(model, 'app_label'), model.__name__.lower(), self.obj.pk, slugify(tab_name)
+                )
                 tab = (tab_name, url)
                 if not self.tabs and not self.current_tab:
                     self.current_tab = slugify(tab_name)
@@ -80,8 +99,10 @@ class ModelPanel(RequestComponent):
                     self.tabs.append(tab)
 
             if not tab_name or slugify(tab_name) == self.current_tab or self.as_pdf:
-
-                fieldset_dict = dict(title=title or 'Dados Gerais', tab_name=tab_name, fields=[], paginators=[], drop_down=drop_down, image=None)
+                fieldset_dict = dict(
+                    title=title or 'Dados Gerais', tab_name=tab_name, fields=[],
+                    paginators=[], drop_down=drop_down, image=None
+                )
                 relations = list(fieldset[1].get('relations', []))
                 inlines = list(fieldset[1].get('inlines', []))
 
@@ -114,7 +135,9 @@ class ModelPanel(RequestComponent):
                                 elif hasattr(attr, 'field'):
                                     field = attr.field
                                 if not field or not hasattr(field, 'display') or field.display:
-                                    verbose_name, lookup, sortable, to = get_fiendly_name(model, attr_name, as_tuple=True)
+                                    verbose_name, lookup, sortable, to = get_fiendly_name(
+                                        model, attr_name, as_tuple=True
+                                    )
                                     if to and not should_filter_or_display(self.request, model, to):
                                         continue
                                     attr_names.append(dict(verbose_name=verbose_name, name=attr_name))
@@ -141,11 +164,17 @@ class ModelPanel(RequestComponent):
                     for info in fieldset[1]['extra']:
                         fieldset_dict['extra'].append(info)
 
+                if 'snippet' in fieldset[1]:
+                    fieldset_dict['snippet'] = fieldset[1]['snippet']
+
     def get_active_fieldsets(self):
         return self.fieldsets_without_tab_name + self.fieldsets_with_tab_name
 
+    def add_action(self, label, url, css, icon=None, category=None):
+        self.dropdown.add_action(label, url, 'class-action {}'.format(css), icon, category)
 
-class IconPanel(RequestComponent):
+
+class IconPanel(Component):
 
     def __init__(self, request, title, icon=None):
         super(IconPanel, self).__init__('iconpanel', request)
@@ -158,14 +187,14 @@ class IconPanel(RequestComponent):
         self.items.append(item)
 
 
-class ShortcutPanel(RequestComponent):
+class ShortcutPanel(Component):
 
     def __init__(self, request):
-        from djangoplus.cache import loader
+        from djangoplus.cache import CACHE
         super(ShortcutPanel, self).__init__('shortcutpanel', request)
         self.items = []
 
-        for model, add_shortcut in loader.icon_panel_models:
+        for model, add_shortcut in CACHE['ICON_PANEL_MODELS']:
 
             if type(add_shortcut) == bool:
                 add_model = add_shortcut
@@ -177,10 +206,12 @@ class ShortcutPanel(RequestComponent):
             if add_model or request.user.is_superuser:
                 if permissions.has_add_permission(request, model):
                     self.add_model(model)
-        for item in loader.views:
+        for item in CACHE['VIEWS']:
             if item['add_shortcut']:
                 if permissions.check_group_or_permission(request, item['can_view']):
-                    self.add(item['icon'], item['title'], None, item['url'], item['can_view'], item['style'])
+                    self.add(
+                        item['icon'], item['verbose_name'], None, item['url'], item['can_view'], item['style']
+                    )
 
     def add(self, icon, description, count=None, url=None, perm_or_group=None, style='ajax'):
         if permissions.check_group_or_permission(self.request, perm_or_group):
@@ -201,19 +232,15 @@ class ShortcutPanel(RequestComponent):
             self.add_model(model)
 
 
-class CardPanel(RequestComponent):
-
-    class Media:
-        pass
-        # css = {'all': ('/static/css/cardpanel.css',)}
+class CardPanel(Component):
 
     def __init__(self, request):
         super(CardPanel, self).__init__('cardpanel', request)
 
         self.items = []
 
-        from djangoplus.cache import loader
-        for model, list_shortcut in loader.card_panel_models:
+        from djangoplus.cache import CACHE
+        for model, list_shortcut in CACHE['CARD_PANEL_MODELS']:
 
             if type(list_shortcut) == bool:
                 add_model = list_shortcut
@@ -226,12 +253,12 @@ class CardPanel(RequestComponent):
                 if permissions.has_list_permission(request, model):
                     self.add_model(model)
 
-                if model in loader.subsets:
+                if model in CACHE['SUBSETS']:
                     icon = get_metadata(model, 'icon')
                     title = get_metadata(model, 'verbose_name_plural')
                     app_label = get_metadata(model, 'app_label')
                     model_name = model.__name__.lower()
-                    for item in loader.subsets[model]:
+                    for item in CACHE['SUBSETS'][model]:
                         can_view = item['can_view']
                         # TODO False
                         if False and permissions.check_group_or_permission(self.request, can_view):
@@ -241,12 +268,7 @@ class CardPanel(RequestComponent):
                             count = qs.count()
                             if count:
                                 url = '/list/{}/{}/{}/'.format(app_label, model_name, attr_name)
-                                self.add(icon, title, count, url, '', None, item['title'])
-
-        for item in loader.views:
-            if False:  # TODO False
-                if permissions.check_group_or_permission(request, item['can_view']):
-                    self.add(item['icon'], item['menu'], None, item['url'], '', item['can_view'], item['style'])
+                                self.add(icon, title, count, url, '', None, item['verbose_name'])
 
     def add(self, icon, title, count=None, url=None, css='', perm_or_group=None, description=''):
         if permissions.check_group_or_permission(self.request, perm_or_group):
@@ -267,7 +289,22 @@ class CardPanel(RequestComponent):
             self.add(icon, title, None, url, '', permission)
 
 
-class DashboardPanel(RequestComponent):
+class RelationModelPanel(Component):
+    def __init__(self, request, qs, title, relation_name):
+        from djangoplus.utils.relations import Relation
+        super(RelationModelPanel, self).__init__('qmp', request)
+        self.title = title
+        self.items = []
+        self.drop_down = GroupDropDown(request)
+        for obj in qs.order_by('id'):
+            paginator = Relation(obj, relation_name).get_component(self.request, self.as_pdf)
+            self.items.append((obj, paginator))
+
+    def add_action(self, label, url, css, icon=None, category=None):
+        self.drop_down.add_action(label, url, 'class-action {}'.format(css), icon, category)
+
+
+class DashboardPanel(Component):
 
     def __init__(self, request):
 
@@ -278,43 +315,28 @@ class DashboardPanel(RequestComponent):
         self.right = []
         self.bottom = []
 
-        from djangoplus.cache import loader
-        for model in loader.subsets:
-            icon = get_metadata(model, 'icon', 'fa-bell-o')
-            title = get_metadata(model, 'verbose_name_plural')
-            app_label = get_metadata(model, 'app_label')
-            model_name = model.__name__.lower()
-            for item in loader.subsets[model]:
-                description = item['title']
-                can_view = item['can_view']
-                notify = item['notify']
-                if notify and permissions.check_group_or_permission(request, can_view):
-                    attr_name = item['function'].__func__.__name__
-                    qs = model.objects.all(request.user)
-                    qs = getattr(qs, attr_name)()
-                    count = qs.count()
-                    if count:
-                        url = '/list/{}/{}/{}/'.format(app_label, model_name, attr_name)
-                        notification_panel = NotificationPanel(request, title, count, url, description, icon)
-                        self.right.append(notification_panel)
+    def add(self, component, position):
+        if position == 'top':
+            self.top.append(component)
+        elif position == 'center':
+            self.center.append(component)
+        elif position == 'left':
+            self.left.append(component)
+        elif position == 'right':
+            self.right.append(component)
+        elif position == 'bottom':
+            self.bottom.append(component)
 
-        icon_panel = ShortcutPanel(request)
-        card_panel = CardPanel(request)
+    def add_widget(self, obj, item):
+        model = item['model']
+        title = item['verbose_name']
+        func_name = item['function']
+        dashboard = item['dashboard']
+        list_display = item.get('list_display')
+        link = item['link']
 
-        self.top.append(icon_panel)
-        self.center.append(card_panel)
-
-        for item in loader.subset_widgets:
-
-            model = item['model']
-            title = item['title']
-            function = item['function']
-            dashboard = item['dashboard']
-            formatter = item['formatter']
-            list_display = item.get('list_display')
-            link = item['link']
-
-            l = []
+        l = []
+        if dashboard:
             if type(dashboard) == dict:
                 for position, group_names in list(dashboard.items()):
                     group_names = type(group_names) == tuple and group_names or (group_names,)
@@ -323,69 +345,125 @@ class DashboardPanel(RequestComponent):
                 l.append((dashboard, item['can_view']))
 
             for position, group_names in l:
-                if permissions.check_group_or_permission(request, group_names, ignore_superuser=True):
-                    qs = model.objects.all(request.user)
-                    f_return = getattr(qs, function)()
-                    html = ''
+                if permissions.check_group_or_permission(self.request, group_names, ignore_superuser=True):
+                    func = getattr(obj, func_name)
+                    f_return = execute_and_format(self.request, func)
 
-                    if type(f_return) in (int, Decimal):
-                        verbose_name = get_metadata(model, 'verbose_name_plural')
-                        icon = get_metadata(model, 'icon')
-                        panel = NumberPanel(request, verbose_name, f_return, title, icon)
-                        html = str(panel)
+                    if f_return is not None:
+                        if type(f_return) in (int, Decimal):
+                            verbose_name = get_metadata(model, 'verbose_name_plural')
+                            icon = get_metadata(model, 'icon')
+                            panel = NumberPanel(self.request, verbose_name, f_return, title, icon)
+                            html = str(panel)
 
-                    if type(f_return).__name__ == 'QueryStatistics' and not formatter:
-                        formatter = 'statistics'
-
-                    if formatter:
-                        func = loader.formatters[formatter]
-                        if len(func.__code__.co_varnames) == 1:
-                            html = str(func(f_return))
+                        elif hasattr(f_return, 'model'):
+                            template_name = item.get('template')
+                            compact = position in ('left', 'right')
+                            app_label = get_metadata(model, 'app_label')
+                            model_name = model.__name__.lower()
+                            verbose_name_plural = get_metadata(model, 'verbose_name_plural')
+                            if link and verbose_name_plural not in title:
+                                title = '{} {}'.format(verbose_name_plural, title)
+                            url = '/list/{}/{}/{}/'.format(app_label, model_name, func_name)
+                            paginator = Paginator(self.request, f_return, title, readonly=compact,
+                                                  list_display=list_display, list_filter=(), search_fields=(),
+                                                  list_subsets=[func_name], url=link and url or None,
+                                                  template=template_name)
+                            if compact and not paginator.template:
+                                paginator.column_names = paginator.column_names[0:1]
+                            html = str(paginator)
                         else:
-                            html = str(func(f_return, request=self.request, verbose_name=title))
-                    elif hasattr(f_return, 'model'):
-                        compact = position in ('left', 'right')
-                        app_label = get_metadata(model, 'app_label')
-                        model_name = model.__name__.lower()
-                        verbose_name_plural = get_metadata(model, 'verbose_name_plural')
-                        if link:
-                            title = '{} {}'.format(verbose_name_plural, title)
-                        url = '/list/{}/{}/{}/'.format(app_label, model_name, function)
-                        paginator = Paginator(self.request, f_return, title, readonly=compact, list_display=list_display, list_filter=(), search_fields=(), list_subsets=[function], url=link and url or None)
-                        if compact:
-                            paginator.column_names = paginator.column_names[0:1]
-                        html = str(paginator)
+                            html = str(f_return)
+                        self.add(html, position)
 
-                    if position == 'top':
-                        self.top.append(html)
-                    elif position == 'center':
-                        self.center.append(html)
-                    elif position == 'left':
-                        self.left.append(html)
-                    elif position == 'right':
-                        self.right.append(html)
-                    elif position == 'bottom':
-                        self.bottom.append(html)
+    def process_request(self):
+        super(DashboardPanel, self).process_request()
+        for container in (self.top, self.center, self.left, self.right, self.bottom):
+            for component in container:
+                if hasattr(component, 'process_request'):
+                    component.process_request()
 
-        for item in loader.widgets:
-            if permissions.check_group_or_permission(request, item['can_view'], ignore_superuser=True):
-                function = item['function']
+
+class AppDashboard(DashboardPanel):
+
+    template_name = 'dashboardpanel.html'
+
+    def __init__(self, request):
+        super(AppDashboard, self).__init__(request)
+        self.load_widgets()
+
+    def load_widgets(self):
+        from djangoplus.cache import CACHE
+        for model in CACHE['SUBSETS']:
+            notification_panels = {}
+            icon = get_metadata(model, 'icon', 'fa-bell-o')
+            title = get_metadata(model, 'verbose_name_plural')
+            app_label = get_metadata(model, 'app_label')
+            model_name = model.__name__.lower()
+            for item in CACHE['SUBSETS'][model]:
+                description = item['verbose_name']
+                notify = item['notify']
+                if notify is True or notify and permissions.check_group_or_permission(self.request, notify):
+                    attr_name = item['function'].__func__.__name__
+                    qs = model.objects.all(self.request.user)
+                    qs = getattr(qs, attr_name)()
+                    count = qs.count()
+                    if count:
+                        url = '/list/{}/{}/{}/'.format(app_label, model_name, attr_name)
+                        if model_name in notification_panels:
+                            notification_panel = notification_panels[model_name]
+                        else:
+                            notification_panel = NotificationPanel(self.request, title, icon)
+                            self.right.append(notification_panel)
+                            notification_panels[model_name] = notification_panel
+                        notification_panel.add(count, url, description)
+                        
+        for model in CACHE['LIST_DASHBOARD']:
+            title = get_metadata(model, 'verbose_name_plural')
+            position = get_metadata(model, 'dashboard')
+            paginator = Paginator(self.request, model.objects.all(self.request.user), title)
+            self.add(paginator, position)
+
+        icon_panel = ShortcutPanel(self.request)
+        card_panel = CardPanel(self.request)
+
+        self.top.append(icon_panel)
+        self.center.append(card_panel)
+
+        for item in CACHE['SUBSET_WIDGETS']:
+            self.add_widget(item['model'].objects.all(self.request.user), item)
+
+        for item in CACHE['WIDGETS']:
+            if permissions.check_group_or_permission(self.request, item['can_view'], ignore_superuser=True):
+                func = item['function']
                 position = item['position']
-                f_return = function(request)
-                html = render_to_string(['{}.html'.format(function.__name__), 'dashboard.html'], f_return, request)
-                if position == 'top':
-                    self.top.append(html)
-                elif position == 'center':
-                    self.center.append(html)
-                elif position == 'left':
-                    self.left.append(html)
-                elif position == 'right':
-                    self.right.append(html)
-                elif position == 'bottom':
-                    self.bottom.append(html)
+                f_return = func(self.request)
+                html = render_to_string(
+                    ['{}.html'.format(func.__name__), 'dashboard.html'], f_return, self.request
+                )
+                self.add(html, position)
 
 
-class NumberPanel(RequestComponent):
+class ModelDashboard(DashboardPanel):
+
+    def __init__(self, request, obj, current_tab=None, parent=None,
+                 fieldsets=None, complete=True, readonly=False, printable=True):
+        super(ModelDashboard, self).__init__(request)
+        self.obj = obj
+        self.model_panel = ModelPanel(
+            request, obj, current_tab=current_tab, parent=parent, fieldsets=fieldsets,
+            complete=complete, readonly=readonly, printable=printable
+        )
+        self.add(self.model_panel, 'center')
+        self.load_widgets()
+
+    def load_widgets(self):
+        from djangoplus.cache import CACHE
+        for item in CACHE['MODEL_WIDGETS'].get(type(self.obj), ()):
+            self.add_widget(self.obj, item)
+
+
+class NumberPanel(Component):
 
     def __init__(self, request, title, number, description, icon=None):
         super(NumberPanel, self).__init__(title, request)
@@ -395,12 +473,14 @@ class NumberPanel(RequestComponent):
         self.icon = icon or 'fa-comment-o'
 
 
-class NotificationPanel(RequestComponent):
+class NotificationPanel(Component):
 
-    def __init__(self, request, title, count, url, description, icon=None):
+    def __init__(self, request, title, icon=None):
         super(NotificationPanel, self).__init__(title, request)
         self.title = title
-        self.count = count
-        self.url = url
-        self.description = description
+        self.items = []
         self.icon = icon or 'fa-bell-o'
+    
+    def add(self, count, url, description):
+        item = dict(count=count, url=url, description=description)
+        self.items.append(item)
